"use strict";
var chalk = require('chalk');
var LOG_LEVELS = {
    error: {
        color: 'red'
    },
    warn: {
        color: 'yellow'
    },
    info: {
        color: 'cyan'
    },
    debug: {
        color: 'white'
    }
};
function log(message, severity) {
    if (severity === void 0) { severity = 'info'; }
    var color = LOG_LEVELS[severity].color || 'cyan';
    if (process.env.SILENT !== 'true') {
        console.log(chalk[color](message));
    }
}
;
var presets = {
    angular: {
        validate: function (message) {
            var MAX_LENGTH = 100;
            var PATTERN = /^(?:fixup!\s*)?(\w*)(\(([\w\$\.\*/-]*)\))?\: (.*)$/;
            var TYPES = {
                feat: true,
                fix: true,
                docs: true,
                style: true,
                refactor: true,
                perf: true,
                test: true,
                chore: true,
                revert: true
            };
            message = message.split('\n').shift();
            if (message.length >= MAX_LENGTH) {
                log("Message is longer than " + MAX_LENGTH + " characters!", 'error');
                return false;
            }
            var match = PATTERN.exec(message);
            if (!match) {
                log("Message does not match \"<type>(<scope>): <subject>\"! was: " + message, 'error');
                return false;
            }
            var type = match[1];
            if (!TYPES.hasOwnProperty(type)) {
                log("'" + type + "' is not an allowed type!", 'error');
                log("Valid types are: " + Object.keys(TYPES).join(', '), 'info');
                return false;
            }
            return true;
        },
        ignorePattern: /^WIP\:/
    },
    atom: {
        validate: function (message) {
            var MAX_LENGTH = 72;
            var EMOJIS = [
                ':art:',
                ':racehorse:',
                ':non-potable_water:',
                ':memo:',
                ':penguin:',
                ':apple:',
                ':checkered_flag:',
                ':bug:',
                ':fire:',
                ':green_heart:',
                ':white_check_mark:',
                ':lock:',
                ':arrow_up:',
                ':arrow_down:',
                ':shirt:'
            ];
            var PATTERN = new RegExp("^(" + EMOJIS.join('|') + ")(?: )(?:.*)$");
            message = message.split('\n').shift();
            if (message.length > MAX_LENGTH) {
                log("Message is longer than " + MAX_LENGTH + " characters!", 'error');
                return false;
            }
            var match = PATTERN.exec(message);
            if (!match) {
                log("Message does not match \"<emoji> <subject>\"! was: " + message, 'error');
                log("Valid emojis are: " + EMOJIS.join(', '), 'info');
                return false;
            }
            return true;
        }
    },
    eslint: {
        validate: function (message) {
            var FIXUP = '(?:fixup!\\s*)?';
            var TAG = '([A-Z][a-z]+)';
            var GH = '(?:(?:[A-Za-z0-9_-]+)\\/(?:[A-Za-z0-9_.-]+))?(?:(?:#|[Gg][Hh]-)\\d+)';
            var MESSAGE = "((?=[A-Z0-9])(?:(?!" + GH + ").)*)";
            var ISSUE = "(\\s\\((?:(?:(?:fixes|refs)\\s" + GH + ")(?:,\\s(?!\\))|\\)))+)?";
            var PATTERN = new RegExp("^" + FIXUP + TAG + ":\\s" + MESSAGE + ISSUE + "$");
            message = message.split('\n').shift();
            var match = PATTERN.exec(message);
            if (!match) {
                log('Message does not match "Tag: Message (fixes #1234)".', 'error');
                log("Given: \"" + message + "\".", 'info');
                return false;
            }
            var matches = match.filter(function (str) { return str ? true : false; }).map(function (str) { return str.trim(); });
            var TAGS = ['Fix', 'Update', 'Breaking', 'Docs', 'Build', 'New', 'Upgrade'];
            if (TAGS.indexOf(matches[1]) === -1) {
                log("The word \"" + matches[1] + "\" is not an allowed tag.", 'error');
                log("Valid types are: " + TAGS.join(', ') + ".", 'info');
                return false;
            }
            return true;
        },
        ignorePattern: /^WIP\:/
    },
    ember: {
        validate: function (message) {
            var TAGS = [
                'DOC',
                'FEATURE',
                'BUGFIX',
                'SECURITY'
            ];
            var TAG = "(" + TAGS.join('|') + ")";
            var MESSAGE = '(.*)';
            var GIT_MESSAGE = '(.*)';
            var VALID_CHANNELS = [
                'canary',
                'beta',
                'release'
            ];
            var PATTERN = new RegExp("^\\[" + TAG + " " + MESSAGE + "\\] " + GIT_MESSAGE);
            message = message.trim().split('\n').shift();
            var match = PATTERN.exec(message);
            if (!match) {
                log('Message does not match "[<tag> <message>] <git-message>"', 'error');
                log("Given: \"" + message + "\".", 'info');
                return false;
            }
            var tag = match[1];
            var msg = match[2];
            if (tag === 'DOC' && VALID_CHANNELS.indexOf(msg) === -1) {
                log('Not a valid channel for DOC', 'error');
                log("Valid channels are " + VALID_CHANNELS.join(', '));
                return false;
            }
            return true;
        }
    },
    jquery: {
        validate: function (message) {
            var SUBJECT_MAX_LENGTH = 72;
            var LONG_DESCRIPTION_MAX_LENGTH = 80;
            var SUBJECT_PATTERN = /^(\w*): ([\w\s\S]*[^.])$/;
            var messageParts = message.trim().split('\n').map(function (line) { return line.trim(); });
            var subject = messageParts[0];
            var match = SUBJECT_PATTERN.exec(subject);
            if (subject.length > SUBJECT_MAX_LENGTH) {
                log("Subject is longer than " + SUBJECT_MAX_LENGTH + " characters.", 'error');
                return false;
            }
            if (!match) {
                log('Subject does not match "Component: Short Description".', 'error');
                log("Given: \"" + subject + "\".", 'info');
                return false;
            }
            var isMessageLengthValid = messageParts.slice(1).every(function (part) {
                return part.length < LONG_DESCRIPTION_MAX_LENGTH;
            });
            if (!isMessageLengthValid) {
                log("Line lengths (except the subject one) should be wrapped at " + LONG_DESCRIPTION_MAX_LENGTH + " columns.", 'error');
                return false;
            }
            return true;
        }
    },
    jshint: {
        validate: function (message) {
            var HEADER_LENGTH = 60;
            var LINES_LENGTH = 100;
            var GH = '(?:(?:[A-Za-z0-9_-]+)\\/(?:[A-Za-z0-9_.-]+))?(?:(?:#|[Gg][Hh]-)\\d+)';
            var SHORTDESCR = "((?=[A-Z0-9])(?:(?!" + GH + ").)*)";
            var TITLE = '\\[{2}([A-Z]+)\\]{2}';
            var FIXUP = '(?:fixup!\\s*)?';
            var HEADER_PATTERN = new RegExp("^" + FIXUP + TITLE + "\\s" + SHORTDESCR + "$");
            var lines = message.trim().split('\n').map(function (line) { return line.trim(); });
            var header = lines.shift();
            if (header.length > HEADER_LENGTH) {
                log("Header is longer than " + HEADER_LENGTH + " characters.", 'error');
                return false;
            }
            var match = HEADER_PATTERN.exec(header);
            if (!match) {
                log('Header does not match "[[TYPE]] Short description".', 'error');
                log("Given: \"" + header + "\".", 'info');
                return false;
            }
            var TITLES = ['FIX', 'FEAT', 'DOCS', 'TEST', 'CHORE'];
            if (TITLES.indexOf(match[1]) === -1) {
                log("The word \"" + match[1] + "\" is not an allowed title.", 'error');
                log("Valid titles are: " + TITLES.join(', ') + ".", 'info');
                return false;
            }
            if (lines.length === 0) {
                return true;
            }
            var secondLine = lines.shift();
            if (secondLine.length > 0) {
                log('Second line of commit message must be a blank line.', 'error');
                return false;
            }
            if (!lines.every(function (line) { return line.length <= LINES_LENGTH; })) {
                log("Line lengths (except first) should be wrapped at " + LINES_LENGTH + " columns.", 'error');
                return false;
            }
            return true;
        },
        ignorePattern: /^\[{2}WIP\]{2}/
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = presets;
